int IIR2(int x, int b, int ma, int y)
// x[0] is the current input signal sample, x[1] is t-1, x[2] is t-2
// y[0] is the current output signal sample, y[1] is t-1, y[2] is t-2
// returns the current filtered output sample
// b[0-2] are filter coeffs
// ma[0-2] are the NEGATIVE of filter coeffs
// General form is "Direct Form II Transposed":
// a(1)*y(n) = b(1)*x(n) + b(2)*x(n-1) + ... + b(nb+1)*x(n-nb)
//                       - a(2)*y(n-1) - ... - a(na+1)*y(n-na)
// with ma[2]=-a(2), etc
// Parameters:
// &xx - input sample 25:24 (hi:lo)
// &b 23:22   
// &ma 21:20    
// &y 19:18
  
    .macro mult_acc         ;r31:r30:r24 += r23:r22 * r21:r20
        muls r23, r21		; (signed)ah * (signed)bh
    	add	r31, r0
    	mul	r22, r20		; al * bl
    	add	r24, r0
    	adc	r30, r1
		clr r1
    	adc	r31, r1			; propagate the carry to the hi-byte
    	mulsu   r23, r20    ; (signed)ah * bl
    	add	r30, r0
    	adc	r31, r1
    	mulsu	r21, r22	; (signed)bh * al
    	add	r30, r0
    	adc	r31, r1
    .endm 
    
	mov r26, r24		; copy xx input to X
	mov r27, r25
	push r16			; save GCC state
	push r17

    clr r24             ;clear 24 bit result reg; msb to lsb => r31:r30:r24
    clr r30      
    clr r31
    
    lds  R22, _b1       ;load b1 from RAM
    lds  R23, _b1+1 
    ld   R20, Y         ;load input parameter xx from stack
    ldd  R21, Y+1
    mult_acc            ; b1*xx  
    
    lds  R22, _b2       ;load b2 from RAM
    lds  R23, _b2+1 
    lds  R20, _xn_1     ;load x(n-1) from  RAM
    lds  R21, _xn_1+1
    mult_acc            ; b2*x(n-1) 
    
    lds  R22, _b3       ;load b3 from RAM
    lds  R23, _b3+1 
    lds  R20, _xn_2     ;load x(n-2) from  RAM
    lds  R21, _xn_2+1
    mult_acc            ; b3*x(n-2)
    
    lds  R22, _a2       ;load -a2 from RAM
    lds  R23, _a2+1 
    lds  R20, _yn_1     ;load y(n-1) from  RAM
    lds  R21, _yn_1+1
    mult_acc            ; -a2*y(n-1)  
    
    lds  R22, _a3       ;load -a3 from RAM
    lds  R23, _a3+1 
    lds  R20, _yn_2     ;load y(n-2) from  RAM
    lds  R21, _yn_2+1
    mult_acc            ; -a3*y(n-2)
    
    lds  R20, _xn_1     ;load x(n-1) from  RAM
    lds  R21, _xn_1+1
    sts  _xn_2, r20      ;store x(n-2) to  RAM
    sts  _xn_2+1, R21 
    ld   R20, Y         ;load input parameter xx from stack
    ldd  R21, Y+1
    sts  _xn_1, r20     ;store x(n-1) to  RAM
    sts  _xn_1+1, R21 
    lds  R20, _yn_1     ;load y(n-1) from  RAM
    lds  R21, _yn_1+1
    sts  _yn_2, R20     ;store y(n-2) to  RAM
    sts  _yn_2+1, R21 
    sts  _yn_1, r30     ;store new output as y(n-1) to  RAM
    sts  _yn_1+1, r31 
	
    clr  r1 ; required for GCC
	ret
	
